﻿using iFileProxy.Services;
using Serilog;
using System.Text.Json;
using System.Text.Json.Serialization;

namespace iFileProxy.Config
{
    public class AppConfig
    {
        static readonly Serilog.ILogger _logger = Log.Logger.ForContext<AppConfig>();
        readonly static JsonSerializerOptions options = new()
        {
            ReadCommentHandling = JsonCommentHandling.Skip, // 允许注释
            AllowTrailingCommas = true // 允许尾随逗号
        };

        public void Save(string configPath = "iFileProxy.json")
        {
            var config = JsonSerializer.Serialize(this);
            File.WriteAllText(configPath, config);
        }

        [JsonPropertyName("Download")]
        public DownloadOptions DownloadOptions { get; set; } = new();

        [JsonPropertyName("Security")]
        public SecurityOptions SecurityOptions { get; set; } = new();

        [JsonPropertyName("Database")]
        public Database Database { get; set; }

        [JsonPropertyName("StreamProxy")]
        public StreamProxyOptions StreamProxyOptions { get; set; } = new();

        public static AppConfig GetCurrConfig(string configPath = "iFileProxy.json")
        {
            if (File.Exists(configPath))
            {
                try
                {
                    return JsonSerializer.Deserialize<AppConfig>(File.ReadAllText(configPath),options);
                }
                catch (Exception e)
                {
                    _logger.Fatal(e,"Config Parse Error!");
                    throw;
                }
            }
            else
                _logger.Fatal($"Config File: {configPath} not exists!");
            return null;
        }
        public static void CheckAppConfig(IServiceProvider serviceProvider)
        {
            AppConfig? c = GetCurrConfig();
            if (c != null)
            {
                if (!Directory.Exists(c.DownloadOptions.SavePath))
                {
                    _logger.Warning($"下载路径不存在, 尝试创建文件夹...");
                    Directory.CreateDirectory(c.DownloadOptions.SavePath);
                    _logger.Warning($"succ.");
                }

                if (!File.Exists(c.DownloadOptions.Aria2cPath))
                {
                    _logger.Fatal("Aria2c 路径错误!");
                    Environment.Exit(1);
                }

                var databaseHelper = serviceProvider.GetRequiredService<DatabaseGateService>();
                if (databaseHelper.TestDbConfig())
                {
                    _logger.Information("数据库配置验证成功.");
                }
            }
            else
            {
                _logger.Error("配置为空!");
                Environment.Exit(1);
            }
        }

    }
    public class DownloadOptions
    {
        public string SavePath { get; set; } = "./proxy_tmp/";
        public uint ThreadNum { get; set; } = 1;
        public long MaxAllowedFileSize { get; set; }
        public uint MaxParallelTasks { get; set; } = 4;
        public uint MaxQueueLength {get; set; } = 60;
        public string Aria2cPath { get; set; } = "./bin/aria2c";
        public int CacheLifetime { get; set; } = 3600;
    }
    public class SecurityOptions
    {
        public List<string> BlockedHost { get; set; } = [];
        public List<string> BlockedFileName { get; set; } = [];
        public List<string> BlockedClientIP { get; set; } = [];
        public List<string> BlockedKeyword { get; set; } = [];
        public List<string> RoutesToTrack { get; set; } = [];
        public int DailyRequestLimitPerIP { get; set; } = -1;
        public bool AllowDifferentIPsForDownload { get; set; } = true;
        public bool AllowStreamTransferOrZeroSize { get; set; } = false;
        public bool EnableUserRegistration { get; set; } = true;
    }

    public partial class Database
    {
        [JsonPropertyName("MaxConnectionPoolSize")]
        public uint MaxConnectionPoolSize { get; set; } = 50;

        [JsonPropertyName("ConnectionLifeTime")]
        public uint ConnectionLifeTime { get; set; } = 120;

        [JsonPropertyName("Common")]
        public Common Common { get; set; }

        [JsonPropertyName("Databases")]
        public List<DB> Databases { get; set; }
    }

    public partial class Common
    {
        [JsonPropertyName("Host")]
        public string Host { get; set; } = string.Empty;

        [JsonPropertyName("Port")]
        public uint Port { get; set; } = 3306;

        [JsonPropertyName("User")]
        public string User { get; set; } = string.Empty;

        [JsonPropertyName("Password")]
        public string Password { get; set; } = string.Empty;
    }
    public partial class DB
    {
        [JsonIgnore(Condition = JsonIgnoreCondition.WhenWritingNull)]
        [JsonPropertyName("Host")]
        public string? Host { get; set; }

        [JsonIgnore(Condition = JsonIgnoreCondition.WhenWritingNull)]
        [JsonPropertyName("Port")]
        public uint? Port { get; set; } = null;

        [JsonIgnore(Condition = JsonIgnoreCondition.WhenWritingNull)]
        [JsonPropertyName("User")]
        public string? User { get; set; }

        [JsonIgnore(Condition = JsonIgnoreCondition.WhenWritingNull)]
        [JsonPropertyName("Password")]
        public string? Password { get; set; }

        [JsonPropertyName("DatabaseName")]
        public string? DatabaseName { get; set; }

        [JsonPropertyName("Description")]
        public string? Description { get; set; }
    }

    public class StreamProxyOptions
    {
        /// <summary>
        /// 文件大小限制（字节）
        /// </summary>
        public long SizeLimit { get; set; } = 1024L * 1024L * 1024L * 10;  // 默认10GB
    }
}
