using iFileProxy.Config;
using iFileProxy.Models;
using Microsoft.AspNetCore.Mvc;
using Serilog;
using System.Web;
using System.Net;
using iFileProxy.Helpers;
using iFileProxy.Services;  // 用于 URL 解码

namespace iFileProxy.Controllers
{
    [Route("/")]
    public class StreamProxyController(IHttpClientFactory httpClientFactory, AppConfig appConfig, DatabaseGateService dbGateService) : ControllerBase
    {
        private readonly IHttpClientFactory _httpClientFactory = httpClientFactory;
        private long SizeLimit = appConfig.StreamProxyOptions.SizeLimit;
        private readonly static Serilog.ILogger _logger = Log.Logger.ForContext<StreamProxyController>();

        // 匹配文件代理请求
        [HttpGet("{*proxyUrl}")]
        [HttpPost("{*proxyUrl}")]
        [HttpPut("{*proxyUrl}")]
        [HttpDelete("{*proxyUrl}")]
        [HttpPatch("{*proxyUrl}")]
        public async Task<IActionResult> ProxyRequest(string proxyUrl)
        {
            try
            {
                if (MasterHelper.IsGithubUrl(proxyUrl))
                {
                    if (!proxyUrl.StartsWith("http://") && !proxyUrl.StartsWith("https://"))
                        proxyUrl = "http://" + proxyUrl; // 默认选择Http协议 如果支持HTTPs应该正常应该会被重定向(除非网站管理员没设置)
                }
                else
                    if (!proxyUrl.StartsWith("http://") && !proxyUrl.StartsWith("https://")) // 不带协议头 直接扔进垃圾桶
                    return StatusCode(400, "非法Url! 除GitHubUrl外其他代理请求必须携带协议头!");
                foreach (var keywords in appConfig.SecurityOptions.BlockedKeyword)
                {
                    if (proxyUrl.Contains(keywords, StringComparison.CurrentCulture))
                        return StatusCode((int)HttpStatusCode.Forbidden, "Keyword::Forbidden");
                }
                var t = new Uri(proxyUrl);
            }
            catch (Exception ex)
            {
                _logger.Error("[Stream] 解析下载文件时出现问题: {ex}", ex);
                return StatusCode((int)HttpStatusCode.InternalServerError, "服务故障 请联系开发者反馈");
            }

            if (string.IsNullOrWhiteSpace(proxyUrl))
            {
                return BadRequest("URL cannot be empty.");
            }

            // URL 解码，确保完整的 URL 被正确处理
            proxyUrl = HttpUtility.UrlDecode(proxyUrl);

            // 如果 URL 没有带协议（http:// 或 https://），默认加上 https://
            if (!proxyUrl.StartsWith("http://") && !proxyUrl.StartsWith("https://"))
            {
                proxyUrl = "https://" + proxyUrl; // 默认为 https
            }

            // 如果目标 host 是 github.com，强制使用 https 协议
            var targetUri = new Uri(proxyUrl);
            if (targetUri.Host == "github.com" && !proxyUrl.StartsWith("https://"))
            {
                proxyUrl = "https://" + targetUri.Host + targetUri.PathAndQuery;
            }

            // 获取原始请求的查询字符串并添加到目标 URL 中
            string queryString = Request.QueryString.HasValue ? Request.QueryString.Value : string.Empty;

            // 拼接完整的 URL，包括查询字符串
            string targetUrl = proxyUrl + queryString;

            try
            {
                var client = _httpClientFactory.CreateClient();

                // 创建请求头（转发客户端的请求头，除了 'Host'）
                var requestHeaders = Request.Headers
                    .Where(h => h.Key != "Host") // 排除 'Host' 头部
                    .ToDictionary(h => h.Key, h => h.Value.ToString());

                List<KeyValuePair<string, string>> failedHeaders = [];

                client.DefaultRequestHeaders.Clear();

                // 设置目标主机名
                // client.DefaultRequestHeaders.Host = targetUri.Host;
                foreach (var header in requestHeaders)
                {
                    // 排除反代理的转发头
                    if (header.Key.StartsWith("X-Forwarded") || header.Key.StartsWith("X-Real"))
                        continue;
                    try
                    {
                        client.DefaultRequestHeaders.Add(header.Key, header.Value);
                    }
                    catch (Exception ex)
                    {
                        // 添加失败的头放到后面处理
                        failedHeaders.Add(new(header.Key, header.Value));
                        _logger.Debug($"Http头添加失败: Key: {header.Key} Value: {header.Value} {ex.Message}");
                    }
                }

                foreach (var header in client.DefaultRequestHeaders)
                {
                    _logger.Debug($"[Request Header] Key: {header.Key} Value: {string.Join(",", header.Value)}");
                }

                // 动态获取客户端的 HTTP 请求方法
                var method = Request.Method;

                // 根据客户端请求方法动态代理请求
                HttpRequestMessage requestMessage = new(new HttpMethod(method), targetUrl);

                if (method == HttpMethods.Post || method == HttpMethods.Put || method == HttpMethods.Patch)
                {
                    var content = new StreamContent(Request.Body); // 仅对于 POST/PUT/PATCH 请求，转发请求体
                                                                   // 处理之前添加失败的请求头
                    foreach (var header in failedHeaders)
                    {
                        content.Headers.Add(header.Key, header.Value);
                    }
                    requestMessage.Content = content;
                }

                // 发送请求并获取响应
                using var response = await client.SendAsync(requestMessage, HttpCompletionOption.ResponseHeadersRead);
                foreach (var header in response.Headers)
                {
                    _logger.Debug($"[Response Header] Key: {header.Key} Value: {string.Join(",", header.Value)}");
                }

                // 确保响应成功
                if (!response.IsSuccessStatusCode)
                {
                    return StatusCode((int)response.StatusCode, await response.Content.ReadAsStringAsync());
                }

                // 获取服务器响应的文件大小
                var contentLength = response.Content.Headers.ContentLength;

                // 如果文件超过大小限制，返回错误
                if (contentLength.HasValue && contentLength.Value > SizeLimit)
                {
                    return StatusCode(413, "File is too large to download.");
                }

                // 设置响应头：告诉浏览器这是一个文件下载
                var fileName = HttpUtility.UrlEncode(Path.GetFileName(new Uri(targetUrl).LocalPath)); // 对文件名进行编码 防止报错
                Response.Headers.Append("Content-Disposition", $"attachment; filename={fileName}");
                Response.ContentType = response.Content.Headers.ContentType?.ToString() ?? "application/octet-stream";

                if (contentLength > 0)
                    Response.ContentLength = contentLength;

                await dbGateService.AddTaskInfoDataAsync(new TaskInfo
                {
                    FileName = fileName,
                    Size = contentLength ?? -1L,
                    ClientIp = MasterHelper.GetClientIPAddr(HttpContext),
                    AddTime = DateTime.UtcNow,
                    UpdateTime = DateTime.UtcNow,
                    Status = TaskState.Stream,
                    TaskId = Guid.NewGuid().ToString(),
                    Url = proxyUrl,
                    Tag = "STREAM"
                });

                // 流式转发文件
                using (var stream = await response.Content.ReadAsStreamAsync())
                {
                    await stream.CopyToAsync(Response.Body);
                }

                return new EmptyResult(); // 控制器返回空结果，响应已经在流中完成
            }
            catch (Exception ex)
            {
                // 捕获异常并返回错误
                return StatusCode(500, $"An error occurred while proxying the request: {ex.Message}");
            }
        }
    }
}
