using iFileProxy.Helpers;
using iFileProxy.Models;
using Microsoft.IdentityModel.Tokens;
using System.IdentityModel.Tokens.Jwt;
using System.Security.Claims;
using System.Text;

namespace iFileProxy.Middlewares
{
    public class JwtMiddleware(RequestDelegate next, IConfiguration configuration)
    {
        private readonly RequestDelegate _next = next;
        private readonly IConfiguration _configuration = configuration;

        public async Task InvokeAsync(HttpContext context)
        {
            var token = context.Request.Headers.Authorization.FirstOrDefault()?.Split(" ").Last();
            var fingerprint = context.Request.Headers["X-Device-Fingerprint"].FirstOrDefault();
            var userAgent = context.Request.Headers.UserAgent.FirstOrDefault();
            var ip = MasterHelper.GetClientIPAddr(context);

            if (token != null)
            {
                try
                {
                    var tokenHandler = new JwtSecurityTokenHandler();
                    var key = Encoding.ASCII.GetBytes(_configuration["Jwt:Key"]);

                    // 验证token
                    tokenHandler.ValidateToken(token, new TokenValidationParameters
                    {
                        ValidateIssuerSigningKey = true,
                        IssuerSigningKey = new SymmetricSecurityKey(key),
                        ValidateIssuer = true,
                        ValidateAudience = true,
                        ValidIssuer = _configuration["Jwt:Issuer"],
                        ValidAudience = _configuration["Jwt:Audience"],
                        ClockSkew = TimeSpan.Zero
                    }, out SecurityToken validatedToken);

                    var jwtToken = (JwtSecurityToken)validatedToken;

                    // 验证客户端信息
                    var tokenFingerprint = jwtToken.Claims.First(x => x.Type == "fingerprint").Value;
                    var tokenUserAgent = jwtToken.Claims.First(x => x.Type == "userAgent").Value;
                    var tokenIp = jwtToken.Claims.First(x => x.Type == "ip").Value;

                    // 如果客户端信息不匹配，则拒绝访问
                    if (fingerprint != tokenFingerprint ||
                        userAgent != tokenUserAgent ||
                        ip != tokenIp && !IsLocalNetwork(ip, tokenIp))  // 允许本地网络IP变化
                    {
                        context.Response.StatusCode = 401;
                        await context.Response.WriteAsJsonAsync(new CommonRsp { Message = "环境发生变化, 请重新验证", Retcode = 401 });
                        return;
                    }

                    var userId = jwtToken.Claims.First(x => x.Type == ClaimTypes.NameIdentifier).Value;
                    context.Items["User"] = userId;
                }
                catch
                {
                    // token验证失败
                }
            }

            await _next(context);
        }

        private static bool IsLocalNetwork(string? ip1, string? ip2)
        {
            if (string.IsNullOrEmpty(ip1) || string.IsNullOrEmpty(ip2)) return false;

            // 检查是否都是内网IP
            return ip1.StartsWith("192.168.") && ip2.StartsWith("192.168.") ||
                   ip1.StartsWith("10.") && ip2.StartsWith("10.") ||
                   ip1.StartsWith("172.") && ip2.StartsWith("172.");
        }
    }
}