using iFileProxy.Config;
using iFileProxy.Helpers;
using iFileProxy.Handlers;
using iFileProxy.Services;
using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.IdentityModel.Tokens;
using Serilog;
using System.Text;
using MySql.Data.MySqlClient;
using iFileProxy.Middlewares;

namespace iFileProxy
{
    public class Program
    {
        public static void Main(string[] args)
        {
            // 绑定异常捕获事件
            AppDomain.CurrentDomain.UnhandledException += CurrentDomain_UnhandledException;
            SerilogConfig.CreateLogger(args: args);

            CommandLineArgsHelper argsHelper = new (args);

            Console.Write(" "); // 补全日志第一行开头的空白

            CmdArgsHandler.ParseArgs(args);

            var builder = WebApplication.CreateBuilder(args);

            // CORS配置
            builder.Services.AddCors(options =>
            {
                options.AddPolicy("AllowFrontend",
                    builder =>
                    {
                        builder
                            .WithOrigins("http://localhost:3000", "http://admin.gitdl.cn", "https://admin.gitdl.cn", "https://github.linxi.info", "http://github.linxi.info/", "http://localhost:4173")
                            .AllowAnyMethod()
                            .AllowAnyHeader()
                            .AllowCredentials();
                    });
            });

            // Add services to the container
            builder.Services.AddControllers();
            builder.Services.AddEndpointsApiExplorer();
            builder.Services.AddSwaggerGen();

            builder.Host.UseSerilog(logger: Log.Logger);

            // 注入依赖
            builder.Services.AddSingleton(AppConfig.GetCurrConfig());
            builder.Services.AddSingleton<DatabaseGateService>();
            builder.Services.AddSingleton<TaskManager>();
            builder.Services.AddSingleton<Dictionary<string, Dictionary<string, uint>>>();

            // 添加验证服务
            builder.Services.AddScoped<AuthService>();

            builder.Services.AddAuthentication(JwtBearerDefaults.AuthenticationScheme)
                .AddJwtBearer(options =>
                {
                    options.TokenValidationParameters = new TokenValidationParameters
                    {
                        ValidateIssuer = true,
                        ValidateAudience = true,
                        ValidateLifetime = true,
                        ValidateIssuerSigningKey = true,
                        ValidIssuer = builder.Configuration["Jwt:Issuer"],
                        ValidAudience = builder.Configuration["Jwt:Audience"],
                        IssuerSigningKey = new SymmetricSecurityKey(
                            Encoding.UTF8.GetBytes(builder.Configuration["Jwt:Key"]))
                    };
                });

            builder.Services.AddHttpClient();

            var app = builder.Build();
            // 初始化缓存管理服务
            LocalCacheManager localCacheManager = new(app.Services);

            if (!argsHelper.GetBooleanValue("disable-startup-check"))
                // 初始化验证配置文件
                AppConfig.CheckAppConfig(app.Services);

            // 1. 错误处理（放在请求管道的最前面）
            app.UseMiddleware<ErrorHandlerMiddleware>();

            // 2. 配置CORS（要在Routing之前）
            app.UseCors("AllowFrontend");

            // 3. 配置静态文件（要在Routing之前）
            var defaultFilesOptions = new DefaultFilesOptions();
            defaultFilesOptions.DefaultFileNames.Clear();
            defaultFilesOptions.DefaultFileNames.Add("index.html");
            app.UseDefaultFiles(defaultFilesOptions);
            app.UseStaticFiles();

            // 4. 强制使用HTTPS（要在Routing之前）
            app.UseHttpsRedirection();

            // 5. 请求日志记录（在Routing之前）
            app.UseSerilogRequestLogging(options =>
            {
                options.EnrichDiagnosticContext = (diagCtx, httpCtx) =>
                {
                    diagCtx.Set("ClientIp", MasterHelper.GetClientIPAddr(httpCtx));
                    diagCtx.Set("contentType", httpCtx.Request.ContentType);
                    diagCtx.Set("queryString", httpCtx.Request.QueryString);
                };
            });

            // 6. 路由配置
            app.UseRouting();

            // 7. 限制访问（IP访问限制，通常在认证之前）
            app.UseMiddleware<IPAccessLimitMiddleware>();

            // 8. 身份验证与授权
            app.UseAuthentication();
            app.UseAuthorization();

            // 9. 配置JWT验证
            app.UseMiddleware<JwtMiddleware>();

            // 10. 映射控制器
            app.MapControllers();

            // 11. 启动应用
            var dbGateService = app.Services.GetRequiredService<DatabaseGateService>();
            SerilogConfig.CreateLogger(dbGateService,args);

            if (!argsHelper.GetStringValue("url").IsNullOrEmpty())
                app.Run(argsHelper.GetStringValue("url"));
            else
                app.Run();
        }


        private static void CurrentDomain_UnhandledException(object sender, UnhandledExceptionEventArgs e)
        {
            var logger = Log.ForContext<AppDomain>();
            Exception exc = (Exception)e.ExceptionObject;

            // 直接记录原始异常信息，不重新抛出
            if (exc is MySqlException mySqlEx)
            {
                if (mySqlEx.Message.Contains("is not allowed to connect to this MySQL server", StringComparison.CurrentCulture))
                {
                    logger.Fatal( "远程主机不允许你连接到该数据库, 请检查数据库服务端是否将本机IP加白! ");
                }
                else if (mySqlEx.Message.Contains("Unable to connect to any of the specified MySQL hosts"))
                {
                    logger.Fatal("无法连接到远程数据库, 因为远程主机不可达, 请检查你的网络或数据库配置!!");
                }
            }
            else if (exc is IOException ioEx)
            {
                if (ioEx.Message.Contains("address already in use."))
                {
                    logger.Fatal("程序服务端口被占用!!!");
                }
            }

            // 记录所有未处理的异常信息
            logger.Fatal(exc, "未处理的异常: {ExceptionMessage}", exc.Message);

            // 优雅退出应用程序
            Environment.Exit(1);
        }
    }
}
