﻿using iFileProxy.Config;
using iFileProxy.Helpers;
using iFileProxy.Models;
using Serilog;
using System.Diagnostics;
using System.Text.Json;

namespace iFileProxy.Services
{
    /// <summary>
    /// 下载任务管理器
    /// </summary>
    public class TaskManager
    {
        // 定义事件
        public event EventHandler<TaskInfo>? TaskCompleted;

        protected virtual void OnTaskCompleted(TaskInfo taskInfo)
        {
            EventHandler<TaskInfo>? handler = TaskCompleted; // 创建事件的副本
            handler?.Invoke(this, taskInfo);
        }


        // 事件处理程序：任务完成后调度下一任务
        private void HandleTaskCompleted(object? sender, TaskInfo taskInfo)
        {
            _logger.Debug($"[TaskId: {taskInfo.TaskId}] Download Successful.");
            _logger.Information($"Running Task Num: {_runningTasks.Count}");
            _logger.Information($"Queue Task Num: {_pendingTasks.Count}");

            // 如果先前获取的大小不正确 则现在更新文件大小
            if (taskInfo.Size <= 0)
            {
                if (MasterHelper.CheckDownloadFileExists(taskInfo.FileName))
                { 
                    taskInfo.Size =MasterHelper.GetDownloadFileSize(taskInfo.FileName);

                    _dbGateService.UpdateFieldsData("size", taskInfo.TaskId, taskInfo.Size);
                }
            }


            // 等待队列中有内容并且当前正在运行的任务小于最大并行任务
            if (_pendingTasks.Count > 0 && _runningTasks.Count < _appConfig.DownloadOptions.MaxParallelTasks)
            {
                lock (_taskLock) // 线程安全
                {
                    int add_task_num = (int)(_appConfig.DownloadOptions.MaxParallelTasks - _runningTasks.Count);
                    if (add_task_num > _pendingTasks.Count)
                        add_task_num = _pendingTasks.Count;
                    for (int i = 0; i < add_task_num; i++)  // 运行的任务中不足最大并行数并且有正在队列的task时候添加足够多的任务
                    {
                        // 获取下一个任务信息
                        TaskInfo nextTask = _pendingTasks.Dequeue();
                        Task.Run(() => StartTaskAsync(nextTask)).ConfigureAwait(false);
                    }
                }
            }
        }


        private readonly static Serilog.ILogger _logger = Log.Logger.ForContext<TaskManager>();
        private readonly AppConfig? _appConfig = AppConfig.GetCurrConfig("iFileProxy.json");
        private readonly DatabaseGateService _dbGateService;
        private Dictionary<string, TaskInfo> _runningTasks = [];
        private Queue<TaskInfo> _pendingTasks = new();
        private readonly object _taskLock = new();

        public TaskManager(IServiceProvider serviceProvider)
        {
            _logger.Information("Initializing TaskManager...");
            if (_appConfig != null)
                _dbGateService = serviceProvider.GetRequiredService<DatabaseGateService>();
            else
            {
                _logger.Fatal($"Failed to load application configuration");
                Environment.Exit(1);
            }

            // 绑定任务完成事件的处理程序
            TaskCompleted -= HandleTaskCompleted;
            TaskCompleted += HandleTaskCompleted;

            _logger.Information("TaskManager init succ.");
        }
        /// <summary>
        /// 添加一个新的下载任务
        /// </summary>
        /// <param name="c">HttpContext</param>
        /// <returns></returns>
        public async Task<TaskAddState> AddTask(HttpContext c)
        {
            string? clientIp = MasterHelper.GetClientIPAddr(c);
            string? t_url = c.Request.Query["url"].FirstOrDefault() ?? c.Request.Form["url"].FirstOrDefault();

            foreach (var keywords in _appConfig.SecurityOptions.BlockedKeyword) {
                if (t_url.Contains(keywords, StringComparison.CurrentCulture))
                    return TaskAddState.ErrKeywordForbidden;
            }

            bool queue_task = false;

            // 如果当前并行任务量已经达到设定并行任务和列队上限
            if (_appConfig.DownloadOptions.MaxParallelTasks != 0 && _runningTasks.Count >= _appConfig.DownloadOptions.MaxParallelTasks)
            {
                if (_pendingTasks.Count >= _appConfig.DownloadOptions.MaxQueueLength)
                    return TaskAddState.ErrMaxParallelTasksLimit;
                else
                    queue_task = true;
            }

            if (_runningTasks.Values.Any(x => x.Url == t_url))
                return TaskAddState.ErrUrlRepeat;

            if (!MasterHelper.CheckUrlIsValid(t_url))
                return TaskAddState.ErrUrlInvalid;

            if (_appConfig.SecurityOptions.BlockedHost.IndexOf(new Uri(t_url).Host) != -1)
                return TaskAddState.ErrTargetHostForbidden;

            DownloadFileInfo? fileInfo = FileDownloadHelper.GetDownloadFileInfo(t_url);
            if (fileInfo == null)
                return TaskAddState.ErrGetFileInfo;

            if (fileInfo.Size > _appConfig.DownloadOptions.MaxAllowedFileSize)
                return TaskAddState.ErrFileSizeLimit;

            if (fileInfo.Size <= 0 && !_appConfig.SecurityOptions.AllowStreamTransferOrZeroSize)
                return TaskAddState.ErrDisabledStreamTransferOrZeroSize;

            if (_appConfig.SecurityOptions.BlockedFileName.IndexOf(fileInfo.FileName) != -1)
                return TaskAddState.ErrFileNameForbidden;

            TaskInfo taskInfo = new()
            {
                Url = t_url,
                TaskId = Guid.NewGuid().ToString(),
                AddTime = DateTime.Now,
                FileName = fileInfo.FileName,
                ClientIp = clientIp,
                Size = fileInfo.Size,
                Status = TaskState.NoInit,
                UpdateTime = DateTime.Now
            };

            // 如果是等待中任务或者列队不是空
            if (queue_task || _pendingTasks.Count != 0) // 判断一下队列长度 防止被插队
            {
                lock(_taskLock)
                if (_pendingTasks.Count >= _appConfig.DownloadOptions.MaxQueueLength)
                    return TaskAddState.ErrQueueLengthLimit;

                _pendingTasks.Enqueue(taskInfo); // 加入等待队列
                var status = await AddTaskInfoToDb(taskInfo,true);
                _logger.Information($"[TaskId: {taskInfo.TaskId}] Queuing...");
                if (status != TaskAddState.Success) { return status; }
                return TaskAddState.Pending;
            }

            if (MasterHelper.CheckDownloadFileExists(taskInfo.FileName))
            {
                var r = _dbGateService.QueryTaskInfo(taskInfo.FileName, taskInfo.Url, taskInfo.Size, TaskState.End);
                if (r != null)
                {
                    taskInfo.Status = TaskState.Cached;
                    taskInfo.Hash = r.Hash;
                    taskInfo.Size = r.Size;
                    taskInfo.AddTag($"REDIRECT:{r.TaskId}");
                }
            }

            return await AddTaskInfoToDb(taskInfo);

        }


        /// <summary>
        /// 添加任务信息到数据库并开始任务
        /// </summary>
        /// <param name="taskInfo">任务信息</param>
        /// <param name="queuing">是否正在排队 若正在排队则不开始任务 等待任务调度</param>
        /// <returns></returns>
        public async Task<TaskAddState> AddTaskInfoToDb(TaskInfo taskInfo, bool queuing = false)
        {
            if (await _dbGateService.AddTaskInfoDataAsync(taskInfo))
            {
                if (!queuing) // 如果不是正在排队的任务
                {
                    if (taskInfo.Status != TaskState.Cached)
                    {
                        StartTaskAsync(taskInfo);
                        taskInfo.Status = TaskState.Running;
                    }
                    _logger.Debug($"[TaskId: {taskInfo.TaskId}] Add to Database Successful.");
                }
                else
                {
                    taskInfo.Status = TaskState.Queuing;
                }
                _dbGateService.UpdateTaskStatus(taskInfo);

                return TaskAddState.Success;
            }
            else
                return TaskAddState.ErrDbFail;
        }

        /// <summary>
        /// 开始任务
        /// </summary>
        /// <param name="taskInfo">任务信息</param>
        /// <returns></returns>
        public async Task StartTaskAsync(TaskInfo taskInfo, bool OnTaskCompletedEvent = true)
        {
            if (_runningTasks.ContainsKey(taskInfo.TaskId))
            {
                _logger.Error($"指定的task已经存在!!!");
                return;
            }

            Process aria2c = new()
            {
                StartInfo = new ProcessStartInfo
                {
                    FileName = _appConfig.DownloadOptions.Aria2cPath,
                    WorkingDirectory = _appConfig.DownloadOptions.SavePath,
                    Arguments = $"-x {_appConfig.DownloadOptions.ThreadNum} -s {_appConfig.DownloadOptions.ThreadNum} {taskInfo.Url}",
                    RedirectStandardOutput = true,
                    RedirectStandardError = true,
                    RedirectStandardInput = true,
                    UseShellExecute = false,
                    Environment = { { "TaskId", taskInfo.TaskId } }
                }
            };

            try
            {
                _runningTasks.Add(taskInfo.TaskId, taskInfo);
                aria2c.Start();
                _runningTasks[taskInfo.TaskId].Process = aria2c;
                if (taskInfo.Status != TaskState.Running)
                {
                    taskInfo.Status = TaskState.Running;
                    _dbGateService.UpdateTaskStatus(taskInfo);
                }
                _logger.Information($"[TaskId: {taskInfo.TaskId}] Started.");
                aria2c.BeginOutputReadLine();
                aria2c.BeginErrorReadLine();
                aria2c.OutputDataReceived += Aria2c_OutputDataReceived;
                aria2c.ErrorDataReceived += Aria2c_ErrorDataReceived;

                await aria2c.WaitForExitAsync();
                _runningTasks.Remove(taskInfo.TaskId);

                if (aria2c.ExitCode != 0)
                {
                    _logger.Error($"task: {taskInfo.TaskId} 进程退出状态异常 ExitCode: {aria2c.ExitCode}");
                    taskInfo.Status = TaskState.Error;
                }
                else
                {
                    taskInfo.Status = TaskState.End;
                    _dbGateService.UpdateTaskStatus(taskInfo);
                    taskInfo.Hash = MasterHelper.GetFileHash(Path.Combine(_appConfig.DownloadOptions.SavePath, taskInfo.FileName), FileHashAlgorithm.MD5);
                    _dbGateService.UpdateTaskHash(taskInfo);
                }
            }
            catch (Exception ex)
            {
                _logger.Fatal("task: {taskid} 执行下载任务时候出现致命问题: {ex}",taskInfo.TaskId, ex);
                taskInfo.Status= TaskState.Error;

                _dbGateService.UpdateTaskStatus(taskInfo);
                throw;
            }
            finally
            {
                if (OnTaskCompletedEvent)
                {
                    // 触发任务完成事件
                    OnTaskCompleted(taskInfo);
                }
            }
        }


        private void Aria2c_ErrorDataReceived(object sender, DataReceivedEventArgs e)
        {
            if (e.Data == null || e.Data.Trim() == "")
                return;
            Process c = (Process)sender;
            _logger.Error($"[TaskId: {c.StartInfo.Environment["TaskId"]}] {e.Data}");
        }

        private void Aria2c_OutputDataReceived(object sender, DataReceivedEventArgs e)
        {
            if (e.Data == null || e.Data.Trim() == "")
                return;
            Process c = (Process)sender;
            _logger.Debug($"[TaskId: {c.StartInfo.Environment["TaskId"]}] {e.Data}");
        }

        /// <summary>
        /// 基于IP地址查询下载列表
        /// </summary>
        /// <param name="c"></param>
        /// <returns></returns>
        public List<TaskInfo> GetTaskListByIpAddr(HttpContext c)
        {
            string? clientIp = MasterHelper.GetClientIPAddr(c);
            return JsonSerializer.Deserialize<List<TaskInfo>>(_dbGateService.GetTaskListByIP(clientIp)) ?? [];
        }

        /// <summary>
        /// 获取任务信息
        /// </summary>
        /// <param name="taskId"></param>
        /// <returns></returns>
        public List<TaskInfo> GetTaskInfo(string taskId)
        {
            _logger.Debug(_dbGateService.GetTaskInfoByTid(taskId));
            return JsonSerializer.Deserialize<List<TaskInfo>>(_dbGateService.GetTaskInfoByTid(taskId)) ?? [];
        }

        public Dictionary<string, TaskInfo> GetRunningTasks() => _runningTasks;

        /// <summary>
        /// 获取task在队列中的位置
        /// </summary>
        /// <param name="taskId"></param>
        /// <returns>成功时返回队列中的位置 失败时返回-1</returns>
        public int GetQueuePosition(string taskId)
        {
            int position = -1; // 默认值表示未找到
            int index = 0;

            Queue<TaskInfo> tempQueue = new( _pendingTasks);

            while (tempQueue.Count > 0)
            {
                TaskInfo current = tempQueue.Dequeue();
                if (current.TaskId == taskId)
                {
                    position = index;
                    break;
                }
                index++;
            }

            return position;
        }

        /// <summary>
        /// 获取服务器任务负载信息
        /// </summary>
        /// <returns></returns>
        public ServerTaskLoadInfo GetServerTaskLoadInfo()
        { 
            return new ServerTaskLoadInfo { Queuing = _pendingTasks.Count , Running = _runningTasks.Count};
        }


        public TaskInfo? GetRunningTaskInfo(string taskId)
        {
            if (_runningTasks.TryGetValue(taskId, out var taskInfo))
            {
                return taskInfo;
            }
            return null;
        }

        public void TryKillTask(TaskInfo taskInfo)
        {
            try 
            {
                // 检查任务是否在运行中
                if (_runningTasks.TryGetValue(taskInfo.TaskId, out var runningTask))
                {
                    _logger.Information($"正在尝试中断任务: {taskInfo.TaskId}");
                    // 更新任务状态为已取消
                    taskInfo.Status = TaskState.Canceled;
                    _dbGateService.UpdateTaskStatus(taskInfo);

                    // 如果存在进程则kill掉
                    foreach (var p in Process.GetProcesses())
                    { 
                        if (p.Id == taskInfo.Process.Id)
                            p.Kill();
                    }

                    // 从运行任务字典中移除
                    _runningTasks.Remove(taskInfo.TaskId);
                   
                    
                    // 删除未完成的文件
                    var filePath = Path.Combine(_appConfig.DownloadOptions.SavePath, taskInfo.FileName);
                    if (File.Exists(filePath))
                    {
                        File.Delete(filePath);
                        _logger.Debug($"已删除未下载完成的文件: {filePath}");
                    }
                    
                    // 删除aria2c的临时文件
                    if (File.Exists(filePath + ".aria2"))
                    {
                        File.Delete(filePath + ".aria2");
                        _logger.Debug("已删除aria2临时文件");
                    }
                    
                    _logger.Information($"任务 {taskInfo.TaskId} 已中断");
                    OnTaskCompleted(taskInfo);
                }
                else
                {
                    _logger.Warning($"任务 {taskInfo.TaskId} 不在运行状态，无需中断");
                }
            }
            catch (Exception ex)
            {
                _logger.Error("中断任务时发生错误: {ex}",ex);
                
                throw;
            }
        }

        /// <summary>
        /// 获取任务的进程详细信息
        /// </summary>
        /// <param name="taskId">任务ID</param>
        /// <returns>进程详细信息，如果任务不存在或未运行则返回null</returns>
        public ProcessInfo? GetProcessInfo(string taskId)
        {
            try
            {
                if (!_runningTasks.TryGetValue(taskId, out var taskInfo) || taskInfo.Process == null)
                {
                    return null;
                }

                var process = taskInfo.Process;
                // 刷新进程信息
                process.Refresh();

                var processInfo = new ProcessInfo
                {
                    ProcessId = process.Id,
                    ProcessName = process.ProcessName,
                    StartTime = process.StartTime,
                    RunningTime = (DateTime.Now - process.StartTime).TotalSeconds,
                    ThreadCount = process.Threads.Count,
                    PriorityClass = process.PriorityClass,
                    WorkingDirectory = process.StartInfo.WorkingDirectory,
                    CommandLine = process.StartInfo.Arguments
                };

                // 获取线程信息
                foreach (ProcessThread thread in process.Threads)
                {
                    try
                    {
                        processInfo.Threads.Add(new ThreadInfo
                        {
                            ThreadId = thread.Id,
                            ThreadState = thread.ThreadState,
                            Priority = thread.PriorityLevel
                        });
                    }
                    catch (Exception ex)
                    {
                        _logger.Warning($"获取线程信息失败: {ex.Message}");
                        // 继续处理下一个线程
                        continue;
                    }
                }

                return processInfo;
            }
            catch (Exception ex)
            {
                _logger.Error($"获取进程信息失败: {ex.Message}");
                return null;
            }
        }

        /// <summary>
        /// 重试失败的任务
        /// </summary>
        /// <param name="taskId">任务ID</param>
        /// <returns>重试是否成功</returns>
        public bool RetryTask(string taskId)
        {
            try
            {
                var taskInfo = JsonSerializer.Deserialize<List<TaskInfo>>(_dbGateService.GetTaskInfoByTid(taskId))?[0];
                if (taskInfo == null)
                {
                    _logger.Warning($"任务不存在: {taskId}");
                    return false;
                }

                // 检查任务状态是否为失败状态
                if (taskInfo.Status != TaskState.Error && taskInfo.Status != TaskState.Canceled)
                {
                    _logger.Warning($"任务 {taskId} 状态为 {taskInfo.Status}，不能重试");
                    return false;
                }

                // 重置任务状态
                taskInfo.Status = TaskState.NoInit;
                taskInfo.UpdateTime = DateTime.Now;
                _dbGateService.UpdateTaskStatus(taskInfo);

                lock (_taskLock)
                {
                    // 如果有等待的任务或达到最大并行数，加入队列
                    if (_pendingTasks.Count > 0 || _runningTasks.Count >= _appConfig.DownloadOptions.MaxParallelTasks)
                    {
                        _pendingTasks.Enqueue(taskInfo);
                        taskInfo.Status = TaskState.Queuing;
                        _dbGateService.UpdateTaskStatus(taskInfo);
                        _logger.Information($"任务 {taskId} 已加入等待队列");
                    }
                    else
                    {
                        // 直接启动任务
                        Task.Run(() => StartTaskAsync(taskInfo)).ConfigureAwait(false);
                    }
                }
                
                return true;
            }
            catch (Exception ex)
            {
                _logger.Error($"重试任务失败: {ex.Message}");
                return false;
            }
        }

        /// <summary>
        /// 将任务移动到等待队列最前面
        /// </summary>
        /// <param name="taskId">任务ID</param>
        /// <returns>是否成功</returns>
        public bool PrioritizeTask(string taskId)
        {
            try
            {
                lock (_taskLock)
                {
                    // 如果队列为空，无需处理
                    if (_pendingTasks.Count == 0)
                    {
                        _logger.Warning("等待队列为空，无法进行优先级调整");
                        return false;
                    }

                    // 找到指定任务
                    var tempQueue = new Queue<TaskInfo>();
                    TaskInfo? targetTask = null;

                    while (_pendingTasks.Count > 0)
                    {
                        var task = _pendingTasks.Dequeue();
                        if (task.TaskId == taskId)
                        {
                            targetTask = task;
                        }
                        else
                        {
                            tempQueue.Enqueue(task);
                        }
                    }

                    // 如果找到目标任务，将其放到新队列的最前面
                    if (targetTask != null)
                    {
                        _pendingTasks.Enqueue(targetTask);
                        while (tempQueue.Count > 0)
                        {
                            _pendingTasks.Enqueue(tempQueue.Dequeue());
                        }
                        _logger.Information($"任务 {taskId} 已移至队列首位");
                        return true;
                    }
                    else
                    {
                        // 如果没找到，还原队列
                        while (tempQueue.Count > 0)
                        {
                            _pendingTasks.Enqueue(tempQueue.Dequeue());
                        }
                        _logger.Warning($"未在等待队列中找到任务: {taskId}");
                        return false;
                    }
                }
            }
            catch (Exception ex)
            {
                _logger.Error($"调整任务优先级失败: {ex.Message}");
                return false;
            }
        }

        /// <summary>
        /// 立即执行指定任务
        /// </summary>
        /// <param name="taskId">任务ID</param>
        /// <returns>是否成功</returns>
        public bool ExecuteImmediately(string taskId)
        {
            try
            {
                lock (_taskLock)
                {
                    // 找到指定任务
                    var tempQueue = new Queue<TaskInfo>();
                    TaskInfo? targetTask = null;

                    while (_pendingTasks.Count > 0)
                    {
                        var task = _pendingTasks.Dequeue();
                        if (task.TaskId == taskId)
                        {
                            targetTask = task;
                        }
                        else
                        {
                            tempQueue.Enqueue(task);
                        }
                    }

                    // 还原其他任务到队列
                    while (tempQueue.Count > 0)
                    {
                        _pendingTasks.Enqueue(tempQueue.Dequeue());
                    }

                    // 如果找到目标任务
                    if (targetTask != null)
                    {
                        _logger.Information($"正在立即执行任务: {taskId}");


                        // 启动任务
                        Task.Run(() => StartTaskAsync(targetTask, false)).ConfigureAwait(false);
                        return true;
                    }
                    else
                    {
                        _logger.Warning($"未在等待队列中找到任务: {taskId}");
                        return false;
                    }
                }
            }
            catch (Exception ex)
            {
                _logger.Error($"立即执行任务失败: {ex.Message}");
                return false;
            }
        }


        //public bool DeleteTask(HttpContext c)
        //{

        //}
    }
}
